package net.sf.sido.constraint;

import java.math.BigDecimal;

import com.google.common.base.Predicate;
import com.google.common.base.Predicates;

public final class Constraints {

	public static <T> Predicate<T> and(final Predicate<T> first, final Predicate<T> second) {
		return new Predicate<T>() {

			@Override
			public boolean apply(T input) {
				return first.apply(input) && second.apply(input);
			}
		};
	}

	public static <N extends Number> Predicate<N> gt(BigDecimal min) {
		return new GTConstraint<N>(min);
	}

	public static <N extends Number> Predicate<N> gte(BigDecimal min) {
		return new GTEConstraint<N>(min);
	}

	public static <N extends Number> Predicate<N> lt(BigDecimal max) {
		return new LTConstraint<N>(max);
	}

	public static <N extends Number> Predicate<N> lte(BigDecimal max) {
		return new LTEConstraint<N>(max);
	}

	public static Predicate<String> matches(String regex) {
		return new MatchesConstraint(regex);
	}

	public static Predicate<String> maxlength(int maxlength) {
		return new StringLengthConstraint(maxlength);
	}

	public static <T> Predicate<T> notNull() {
		return Predicates.notNull();
	}

	private Constraints() {
	}

}
