package net.sf.sido.support;

import java.math.BigDecimal;
import java.util.Map;

import net.sf.sido.DOInvalidPropertyTypeException;
import net.sf.sido.DataObject;
import net.sf.sido.Property;
import net.sf.sido.PropertyController;
import net.sf.sido.PropertyMetaInfos;

import org.joda.time.DateTime;
import org.joda.time.Duration;
import org.joda.time.Period;

public class SimplePropertyController<T> implements PropertyController<T> {

	private final Class<T> type;

	public SimplePropertyController(Class<T> type) {
		this.type = type;
	}

	@SuppressWarnings("unchecked")
	@Override
	public T getDefaultValue(boolean deep) {
		if (Boolean.class.isAssignableFrom(type)) {
			return (T) Boolean.FALSE;
		} else if (String.class.isAssignableFrom(type)) {
			return (T) "";
		} else if (Byte.class.isAssignableFrom(type)) {
			return (T) Byte.valueOf((byte) 0);
		} else if (Short.class.isAssignableFrom(type)) {
			return (T) Short.valueOf((short) 0);
		} else if (Integer.class.isAssignableFrom(type)) {
			return (T) Integer.valueOf(0);
		} else if (Long.class.isAssignableFrom(type)) {
			return (T) Long.valueOf(0);
		} else if (Float.class.isAssignableFrom(type)) {
			return (T) Float.valueOf(0);
		} else if (Double.class.isAssignableFrom(type)) {
			return (T) Double.valueOf(0);
		} else if (BigDecimal.class.isAssignableFrom(type)) {
			return (T) BigDecimal.ZERO;
		} else if (DateTime.class.isAssignableFrom(type)) {
			return (T) new DateTime();
		} else if (Period.class.isAssignableFrom(type)) {
			return (T) new Period();
		} else if (Duration.class.isAssignableFrom(type)) {
			return (T) new Duration(new Period());
		} else {
			return null;
		}
	}

	@Override
	public void control(Property<T> property, DataObject o, T value) {
		// Checks the type
		if (!type.isInstance(value)) {
			throw new DOInvalidPropertyTypeException(o, property.getName(), type.getName(), value);
		}
	}

	@Override
	public void fillMetaInfo(Map<String, Object> mi) {
		mi.put(PropertyMetaInfos.ELEMENT_TYPE, type);
	}

}
